//
//  StatusViewController.m
//  Gi2MO Monitor
//
//  Created by cyanide on 2/26/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "ScrollViewController.h"
#import "MainViewController.h"
#import "ImsStatusPageController.h"
#import "themes/MetroStylePanelController.h"
#import "DataController.h"
#import "FlipsideViewController.h"

#import "XMLReader.h"
#import "IdeaManagementStatus.h"

#import "IMS.h"
#import "Idea.h"
#import "Settings.h"

@implementation ScrollViewController

//core data
@synthesize managedObjectContext;
@synthesize dataController;
@synthesize applicationSettings;

//pagination
@synthesize scrollView;
@synthesize pageControl;
@synthesize viewControllers;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
        numberOfPages = 1;
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    
    //set core data properties
    self.dataController = [[DataController alloc] init:self.managedObjectContext];
    
    //-------test scroll view----------
    numberOfPages = [dataController countOfList];
    NSLog(@"Page number: %d", numberOfPages);
    NSMutableArray *controllers = [[NSMutableArray alloc] init];
    for (unsigned i = 0; i < numberOfPages; i++) {
        [controllers addObject:[NSNull null]];
        

        //attempt to resync the views when starting the application (enable this depending on Settings)
        
        if ([applicationSettings.autorefreshType intValue] == 1) //if autorefresh on start enabled
        {
            IMS *ims = [dataController objectInListAtIndex:i];
            IdeaManagementStatus *ideaManagementStatus = [self fetchRDFStatistics: ims.url];
            [dataController updateData: ideaManagementStatus AtIndex: i];
        }
    }
    
    if (numberOfPages==0) //if no instances added put blank panel
    {
        [controllers addObject:[NSNull null]]; 
        numberOfPages = 1;
    }
    self.viewControllers = controllers;
    [controllers release];
    
    //IMS *ims = [dataController objectInListAtIndex:0];
    //NSLog(@"Fetched IMS \"%@\" Status with last idea count: %@", ims.title, ims.hasIdeaCount);
    
    //set gui properties
    scrollView.clipsToBounds = YES;
    scrollView.contentSize = CGSizeMake(scrollView.frame.size.width * numberOfPages, scrollView.frame.size.height);
    scrollView.showsHorizontalScrollIndicator = NO;
    scrollView.showsVerticalScrollIndicator = NO;
    scrollView.scrollsToTop = NO;
    scrollView.delegate = self;
    
     //kNumberOfPages;
    pageControl.numberOfPages = numberOfPages;
    pageControl.currentPage = [applicationSettings.currentPage integerValue];
    
    //test loading all pages in one go
    for (int i = 0; i<numberOfPages; i++)
    {
        [self loadScrollViewWithPage:i];
    }
    
    /* 
    //loading only 2 closest pages
    if (pageControl.currentPage == 0)
    {
        [self loadScrollViewWithPage:0];
        [self loadScrollViewWithPage:1];

    }
    else
    [self.scrollView setContentOffset: CGPointMake(scrollView.frame.size.width * pageControl.currentPage, 0) animated: NO]; */
    if (pageControl.currentPage != 0)
    [self.scrollView setContentOffset: CGPointMake(scrollView.frame.size.width * pageControl.currentPage, 0) animated: NO];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark -
#pragma ScrollView management

- (void)loadScrollViewWithPage:(int)page {
    if (page < 0) return;
    if (page >= numberOfPages) return;

//    ImsStatusPageController *controller = [viewControllers objectAtIndex:page];
    MetroStylePanelController *controller = [viewControllers objectAtIndex:page];
    if ((NSNull *)controller == [NSNull null]) {
        //controller = [ [ImsStatusPageController alloc] initWithNibName:@"ImsStatusPageController" bundle:nil];
        
//        controller = [ [ImsStatusPageController alloc] initWithPageNumber:page];
        controller = [ [MetroStylePanelController alloc] initWithPageNumber:page];
        
        controller.parentScrollViewController = self;
        controller.managedObjectContext = self.managedObjectContext;
        controller.dataController = self.dataController;
        controller.applicationSettings = self.applicationSettings;
        //[[ [TestPageController alloc] initWithNibName:@"TestPageController" bundle:nil] autorelease];
        
        //[ [TestPageController alloc] initWithNibName:@"TestPageController" bundle:nil];
        
        //[[TestPageController alloc] initWithPageNumber:page];
        //[[[StatusViewController alloc] initWithNibName:@"StatusViewController" bundle:nil] autorelease];
        //[[StatusViewController alloc] initWithPageNumber:page];
        
        [viewControllers replaceObjectAtIndex:page withObject:controller];
        [controller release];
    }

    if (nil == controller.view.superview) {
        CGRect frame = scrollView.frame;
        frame.origin.x = frame.size.width * page;
        frame.origin.y = 0;
        controller.view.frame = frame;
        [scrollView addSubview:controller.view];
    }
}

- (void)scrollViewDidScroll:(UIScrollView *)sender {
    if (pageControlUsed) {
        return;
    }
    CGFloat pageWidth = scrollView.frame.size.width;
    int page = floor((scrollView.contentOffset.x - pageWidth / 2) / pageWidth) + 1;
    pageControl.currentPage = page;
    
    [self loadScrollViewWithPage:page - 1];
    [self loadScrollViewWithPage:page];
    [self loadScrollViewWithPage:page + 1];
    
}
- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView {
    pageControlUsed = NO;
}
- (IBAction)changePage:(id)sender {
    int page = pageControl.currentPage;
    [self loadScrollViewWithPage:page - 1];
    [self loadScrollViewWithPage:page];
    [self loadScrollViewWithPage:page + 1];
    CGRect frame = scrollView.frame;
    frame.origin.x = frame.size.width * page;
    frame.origin.y = 0;
    [scrollView scrollRectToVisible:frame animated:YES];
    pageControlUsed = YES;
}

#pragma mark - Flipside View

- (void)flipsideViewControllerDidFinish:(FlipsideViewController *)controller
{
    //------------------------------------
    
    if (controller.listDidChange == YES || currentDiffMode != applicationSettings.diffMode) { //only reload if the sources list changed
        NSLog(@"list changed refresh all panes");    
        //clean previous controls from the scrollView
        for (UIView *view in scrollView.subviews) {
            [view removeFromSuperview];
        }
        //[self.viewControllers release];
        
        //set core data properties
        self.dataController = [[DataController alloc] init:self.managedObjectContext];
        
        //-------test scroll view----------
        numberOfPages = [dataController countOfList];
        NSMutableArray *controllers = [[NSMutableArray alloc] init];
        for (unsigned i = 0; i < numberOfPages; i++) {
            [controllers addObject:[NSNull null]];
        }
        
        if (numberOfPages==0) //if the list was wiped clean and no instances added put a blank panel
        {
            [controllers addObject:[NSNull null]]; 
            numberOfPages = 1;
        }
        
        self.viewControllers = controllers;
        [controllers release];
        
        //set gui properties
        scrollView.pagingEnabled = YES;
        scrollView.contentSize = CGSizeMake(scrollView.frame.size.width * numberOfPages, scrollView.frame.size.height);
        scrollView.showsHorizontalScrollIndicator = NO;
        scrollView.showsVerticalScrollIndicator = NO;
        scrollView.scrollsToTop = NO;
        scrollView.delegate = self;
        
        //kNumberOfPages;
        pageControl.numberOfPages = numberOfPages; //[dataController countOfList];
        //pageControl.currentPage = 0;
        
        //[self loadScrollViewWithPage:0];
        //[self loadScrollViewWithPage:1];
        
        //test loading all pages in one go
        for (int i = 0; i<numberOfPages; i++)
        {
            [self loadScrollViewWithPage:i];
        }
    }
    
    //check if diff counting settings did change
    if (currentDiffMode != applicationSettings.diffMode)
        NSLog(@"Diff settings changed !");
    /*
    int page = pageControl.currentPage; 
    //load 2 closest pages
    [self loadScrollViewWithPage:page - 1];
    [self loadScrollViewWithPage:page];
    [self loadScrollViewWithPage:page + 1];
    */
    //------------------------------------
    [self dismissModalViewControllerAnimated:YES];
}

- (void)showInfo
{    
    FlipsideViewController *controller = [[[FlipsideViewController alloc] initWithNibName:@"FlipsideViewController" bundle:nil] autorelease];
    // FlipsideViewController *controller = [[[FlipsideViewController alloc] initWithStyle: UITableViewStylePlain] autorelease];
    
    controller.delegate = self;
    controller.managedObjectContext = self.managedObjectContext;
    controller.applicationSettings = self.applicationSettings;
    
    controller.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;
    
    //save diff mode before entering settings screen (to compare if something changed after settings are saved)
    currentDiffMode = applicationSettings.diffMode;
    [self presentModalViewController:controller animated:YES];
}

//fetch the RDF and return status object
- (IdeaManagementStatus *) fetchRDFStatistics :(NSString *) url
{
    NSURLRequest* chRequest = [NSURLRequest requestWithURL:[NSURL URLWithString: url] cachePolicy: NSURLRequestReloadIgnoringCacheData timeoutInterval:10];
    NSError* theError;
    NSData* response = [NSURLConnection sendSynchronousRequest:chRequest returningResponse:nil error:&theError]; 
    
    NSXMLParser *xmlParser = [[NSXMLParser alloc] initWithData:response];
    
    //enable namespace parsing
    [xmlParser setShouldProcessNamespaces:YES];
    
    //Initialize the delegate.
    XMLReader *parser = [[XMLReader alloc] initXMLReader];
    
    //Set delegate
    [xmlParser setDelegate:parser];
    
    //Start parsing the XML file.
    BOOL success = [xmlParser parse];
    
    if(success && response!= nil)
    {
        return parser.ideaManagementStatus;
    } 
    else return nil;
}

@end
